# 微信公众号接收回复和消息回复（被动）
## 服务器配置
要接收来自公众号的消息并自动回复，需要搭建一个本地服务器。可以使用python的flask模块，轻松搭建服务器。如：
```
from flask import Flask
app = Flask(__name__)

@app.route('/')
def hello_world():
    return 'Hello, World!'

app.run(host='127.0.0.1', port=5000, debug=True)
```
此时访问http://127.0.0.1:5000/即可看到：

![本地服务器的简单实例](https://upload-images.jianshu.io/upload_images/10171495-f9464fa450f958a7.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)

但在使用微信公众号连接服务器之前首先要验证token，也就是要先执行本地服务器程序，然后

![服务器设置的token](https://upload-images.jianshu.io/upload_images/10171495-87aa726cde941e31.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)

代码如下：
```
# -*- coding:utf-8 -*-
from flask import Flask
from flask import request
import hashlib
app = Flask(__name__)
app.debug = True
@app.route('/wx_flask',methods=['GET','POST'])
def wechat():
    if request.method == 'GET':
        # token
        token = 'luopengting'
        # parameter
        data = request.args
        signature = data.get('signature','')
        timestamp = data.get('timestamp','')
        nonce = data.get('nonce','')
        echostr = data.get('echostr','')
        # dict sort
        list = [token, timestamp, nonce]
        list.sort()

        s = list[0] + list[1] + list[2]
        #sha1
        hascode = hashlib.sha1(s.encode('utf-8')).hexdigest()
        #from wechat, reply echostr
        if hascode == signature:
            return echostr
        else:
            return ""
if __name__ == '__main__':
    app.run(host='0.0.0.0', port=80, debug=True)
```
**运行该token验证程序**，然后将程序里设置的蓝色部分的主页地址和红色部分的token填入下面的设置中，然后提交设置，此时可以看到验证成功。如果未提前运行，则会设置失败。

![填写服务器设置](https://upload-images.jianshu.io/upload_images/10171495-fe8c77c4ce6f32ae.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)

### 接收消息和发送的消息格式
- 接收文本消息
对以下格式的消息进行xml解析，则可以获得ToUserName（微信公众号的id）、FromUserName（用户的openid）和Content（消息内容）
```
<xml>
  <ToUserName>< ![CDATA[toUser] ]></ToUserName>
  <FromUserName>< ![CDATA[fromUser] ]></FromUserName>
  <CreateTime>1348831860</CreateTime>
  <MsgType>< ![CDATA[text] ]></MsgType>
  <Content>< ![CDATA[this is a test] ]></Content>
  <MsgId>1234567890123456</MsgId>
</xml>
```
- 发送前需要转成xml格式
采用如下方法：
```
class TextMsg(Msg):
    def __init__(self, toUserName, fromUserName,content):
        self.__dict = dict()
        self.__dict['ToUserName'] = toUserName
        self.__dict['FromUserName'] = fromUserName
        self.__dict['CreateTime'] =int(time.time())
        self.__dict['Content'] = content

    def send(self):
        XmlForm = """
        <xml>
        <ToUserName><![CDATA[{ToUserName}]]></ToUserName>
        <FromUserName><![CDATA[{FromUserName}]]></FromUserName>
        <CreateTime>{CreateTime}</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[{Content}]]></Content>
        </xml>
        """
        return XmlForm.format(**self.__dict)
```
- 接收并且回复
判断消息类型，并且检测是否有特定关键词。如果是设置用户参数/订阅股票，则更新数据库对应值。并且回复确认。
```
    if rec_msg.MsgType == 'text':
        content = rec_msg.Content.decode('utf-8')
        if content.startswith(u"设置", 0, 2):
            mi.update_threshold(rec_msg.FromUserName, content)
            rep_text_msg = reply.TextMsg(rec_msg.FromUserName, rec_msg.ToUserName,("设置成功 \n %s"%getTime()))
            return rep_text_msg.send()
        elif content.startswith(u"订阅", 0, 3):
            mi.update_stockid()    # 未设置
            rep_text_msg = reply.TextMsg(rec_msg.FromUserName, rec_msg.ToUserName, ("订阅成功 \n %s" % getTime()))
            return rep_text_msg.send()
        else:
            rep_text_msg = reply.TextMsg(rec_msg.FromUserName,rec_msg.ToUserName,"我来学你说：%s \n %s"%(content,getTime()))
            return rep_text_msg.send()
```

## 实例
> wx_reply_test.py

这个程序的运行结果是：如果收到用户发“你好”，则回复“hello!!”；如果用户发其他，则回复同样的话；如果是图片，也回发该图。

```
# -*- coding: utf-8 -*-
# filename:
from flask import Flask, request, make_response
import time, hashlib
import reply
import receive

app = Flask(__name__)

@app.route('/wx_flask', methods=['GET', 'POST'])
def wechat():
    # 微信验证token
    if request.method == 'GET':
        token = 'luopengting'
        query = request.args
        signature = query.get('signature', '')
        timestamp = query.get('timestamp', '')
        nonce = query.get('nonce', '')
        echostr = query.get('echostr', '')
        s = [timestamp, nonce, token]
        s.sort()
        s = ''.join(s)
        if hashlib.sha1(s).hexdigest() == signature:
            return make_response(echostr)
    else:
        rec_msg = receive.parse_xml(request.stream.read())

        if rec_msg.MsgType == 'text':
            content = rec_msg.Content.decode('utf-8')
            if content.startswith(u"你好", 0, 2):
                 rep_text_msg = reply.TextMsg(rec_msg.FromUserName, rec_msg.ToUserName,("hello!! \n %s"%getTime()))
                 return rep_text_msg.send()
            else:
                 rep_text_msg = reply.TextMsg(rec_msg.FromUserName,rec_msg.ToUserName,"复述：%s \n %s"%(content,getTime()))
                 return rep_text_msg.send()
        elif  rec_msg.MsgType =="image":
            rep_img_msg = reply.ImageMsg(rec_msg.FromUserName,rec_msg.ToUserName,rec_msg.MediaId)
            return rep_img_msg.send()
        else:
            return "success"
#获取时间戳
def getTime():
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())

app.run(host='0.0.0.0', port=80, debug=True)
```

## 运行
运行 wx_reply_test.py：

![成功启动服务器](https://upload-images.jianshu.io/upload_images/10171495-68432641ee5e9d62.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)

在微信公众号发消息测试：

![接收消息](https://upload-images.jianshu.io/upload_images/10171495-c0bc8921cf59f4fc.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/800)

![自动回复](https://upload-images.jianshu.io/upload_images/10171495-b9d437117829fe02.jpg?imageMogr2/auto-orient/strip%7CimageView2/2/w/400)